
//  MonthCalendars.swift  Copyright (c) Kari Laitinen

//  http://www.naturalprogramming.com

//  2014-10-13  File created.
//  2016-11-09  Last modification.

//  This program demonstrates:
//    - inheritance
//    - use of standard classes Date, Calendar, etc.
//    - a menu-based CLI (Command Line Interface) application

import Foundation


class  EnglishCalendar
{
   var  this_year    = 2014
   var  this_month   = 10

   var  names_of_months   =

         [ "January", "February", "March", "April",
           "May", "June", "July", "August",
           "September", "October", "November", "December" ]

   var  week_description =

          " Week   Mon  Tue  Wed  Thu  Fri  Sat  Sun"

   init( _ given_year : Int, _ given_month : Int )
   {
      this_year   =  given_year
      this_month  =  given_month
   }

   func get_calendar_year() -> Int
   {
      return this_year
   }

   func get_calendar_month() -> Int
   {
      return this_month
   }


   func increment_calendar_month()
   {
      this_month +=  1

      if  this_month > 12 
      {
         this_month = 1
         this_year +=  1
      }
   }

   func print_calendar()
   {
      var calendar = Calendar.current
      calendar.firstWeekday = 2 // This should be monday

      var date_components = DateComponents()
      date_components.year = this_year
      date_components.month = this_month
      date_components.day = 1
      date_components.hour = 14  // Two o'clock in the afternoon.

      // Setting the date's time to 14 was necessary in order to make
      // the incrementing of the date work properly.

      // In the beginning a_date_in_this_month refers to an NSDate object
      // that contains the first day of this month

      var a_date_in_this_month = calendar.date( from: date_components )!

      let day_of_week_of_first_day  =
                        calendar.component( Calendar.Component.weekday,
                                            from: a_date_in_this_month )


      print(  String( format: "\n\n   %@  %d",  names_of_months[ this_month - 1 ],
                      this_year ) )
      print( "\n"  +  week_description   +  "\n" )

      // Let's find out and print the week number of the first week.

      var week_of_year = calendar.component( Calendar.Component.weekOfYear,
                                             from: a_date_in_this_month )
 
      print( String( format: "%4d  ",  week_of_year ), terminator: "" )

      // The first week of a month is often an incomplete week,
      // i.e., the first part of week belongs to the previous
      // month. In place of the days that bevar to the previous
      // month we print just spaces.

      var day_of_week_index   =  0

      while  day_of_week_index < 6 &&
             day_of_week_index != ( day_of_week_of_first_day - 2 ) 
      {
         print( "     ", terminator: "" )
         day_of_week_index  +=  1
      }


      var day_index = 0

      var month_of_next_day = 0

      repeat
      {
         if  day_of_week_index  >=  7 
         {
            // A new week starts. We must print the week number.

            week_of_year = calendar.component( Calendar.Component.weekOfYear,
                                               from: a_date_in_this_month )

            print( String( format: "\n%4d  ", week_of_year ), terminator: "" )

            day_of_week_index  =  0
         }

         print( String( format: "%5d", ( day_index + 1 ) ), terminator: "" ) 

         // After printing a date we will increment it by one day.
         // There are 24 * 60 * 60 == 86400 seconds in each full day.

         a_date_in_this_month = 
                  a_date_in_this_month.addingTimeInterval( 86400 )

         day_of_week_index  +=  1
         day_index += 1

         // We must check if we already reached the next month.

         month_of_next_day = calendar.component( Calendar.Component.month,
                                                 from: a_date_in_this_month )

      }  while ( this_month == month_of_next_day )

      print( "\n" )
 
   }
}



class  SpanishCalendar  :  EnglishCalendar
{
   override init( _ given_year : Int, _ given_month : Int )
   {
      super.init( given_year, given_month )

      // We'll modify the texts that will be used when this
      // calendar is printed.

      let spanish_names_of_months  =

         [ "Enero", "Febrero", "Marzo", "Abril",
           "Mayo", "Junio", "Julio", "Agosto",
           "Septiembre", "Octubre", "Noviembre", "Diciembre" ]

      let  spanish_week_description  =

           "Semana  Lun  Mar  Mie  Jue  Vie  Sab  Dom"

      names_of_months   =  spanish_names_of_months
      week_description  =  spanish_week_description
   }
}


//  Here begins the main program.


var a_spanish_calendar  = SpanishCalendar( 2016, 10 )

a_spanish_calendar.print_calendar()


var calendar_to_print = EnglishCalendar( 2016, 11 )

calendar_to_print.print_calendar()

var user_selection  =  "????"

print( "\n This program prints calendars. Please, select from"
    +  "\n the following menu by typing in a letter. ", terminator: "" )

while  user_selection  !=  "e" 
{
   print("\n\n   p   Print current calendar."
        +  "\n   n   Print next calendar."
        +  "\n   s   Switch to Spanish calendars."
        +  "\n   e   Exit the program.\n\n   ", terminator: "" )

   user_selection  =  readLine()!

   if  user_selection  ==  "p"
   {
      calendar_to_print.print_calendar()
   }
   else if  user_selection  ==  "n"
   {
      calendar_to_print.increment_calendar_month()

      calendar_to_print.print_calendar()
   }
   else if  user_selection  ==  "s"
   {
      // We'll take the calendar year and month from the old calendar
      // object and use them to create a SpanishCalendar object.

      calendar_to_print =
               SpanishCalendar( calendar_to_print.get_calendar_year(),
                                calendar_to_print.get_calendar_month() )

      calendar_to_print.print_calendar()
   }
}



